@@ -28,7 +28,7 @@ type halign =
 [@@deriving sexp_of]
 
 type t =
-  | Text of Utf8_text.t
+  | Text of String.Utf8.t
   | Fill of Uchar.t * dims
   | Hcat of t * t * dims
   | Vcat of t * t * dims
@@ -41,7 +41,7 @@ let height = function
 ;;
 
 let width = function
-  | Text s -> Utf8_text.width s
+  | Text s -> String.Utf8.length_in_uchars s
   | Fill (_, d) | Hcat (_, _, d) | Vcat (_, _, d) | Ansi (_, _, _, d) -> d.width
 ;;
 
@@ -49,7 +49,7 @@ let uchar_newline = Uchar.of_char '\n'
 
 let rec invariant t =
   match t with
-  | Text s -> assert (not (Utf8_text.mem s uchar_newline))
+  | Text s -> assert (not (String.Utf8.mem s uchar_newline))
   | Fill (_, dims) -> dims_invariant dims
   | Hcat (t1, t2, dims) ->
     dims_invariant dims;
@@ -194,26 +194,29 @@ let text_of_lines lines ~align =
   | _ -> lines |> List.map ~f:(fun line -> Text line) |> vcat ~align
 ;;
 
-let utf8_space = Utf8_text.of_string " "
+let utf8_space = String.Utf8.of_string " "
 
 let word_wrap line ~max_width =
-  Utf8_text.split line ~on:' '
-  |> List.filter ~f:(Fn.non Utf8_text.is_empty)
+  String.Utf8.split line ~on:(Uchar.of_char ' ')
+  |> List.filter ~f:(Fn.non String.Utf8.is_empty)
   |> List.fold ~init:(Fqueue.empty, Fqueue.empty, 0) ~f:(fun (lines, line, len) word ->
-    let n = Utf8_text.width word in
-    let n' = len + 1 + n in
-    if n' > max_width
-    then Fqueue.enqueue lines line, Fqueue.singleton word, n
-    else lines, Fqueue.enqueue line word, n')
+       let n = String.Utf8.length_in_uchars word in
+       let n' = len + 1 + n in
+       if n' > max_width
+       then Fqueue.enqueue lines line, Fqueue.singleton word, n
+       else lines, Fqueue.enqueue line word, n')
   |> (fun (lines, line, _) -> Fqueue.enqueue lines line)
-  |> Fqueue.map ~f:(fun line -> Fqueue.to_list line |> Utf8_text.concat ~sep:utf8_space)
+  |> Fqueue.map ~f:(fun line ->
+       Fqueue.to_list line |> List.intersperse ~sep:utf8_space |> String.Utf8.concat)
   |> Fqueue.to_list
 ;;
 
 let text ?(align = `Left) ?max_width str =
-  let txt = Utf8_text.of_string str in
+  let txt = String.Utf8.of_string str in
   let lines =
-    if Utf8_text.mem txt uchar_newline then Utf8_text.split ~on:'\n' txt else [ txt ]
+    if String.Utf8.mem txt uchar_newline
+    then String.Utf8.split ~on:(Uchar.of_char '\n') txt
+    else [ txt ]
   in
   let lines =
     match max_width with
@@ -235,38 +238,58 @@ let render_abstract t ~write_direct ~line_length =
   let next_i = Array.init (height t) ~f:(fun _ -> 0) in
   let add_char c j =
     let i = next_i.(j) in
-    let num_bytes = Uchar.utf8_byte_length c in
+    let num_bytes = Uchar.Utf8.byte_length c in
     next_i.(j) <- i + num_bytes;
     write_direct c i j ~num_bytes
   in
-  let write_string txt j = Utf8_text.iter txt ~f:(fun uchar -> add_char uchar j) in
-  let rec aux t j_offset =
-    match t with
-    | Text s -> write_string s j_offset
-    | Fill (ch, d) ->
-      for _i = 0 to d.width - 1 do
-        for j = 0 to d.height - 1 do
-          add_char ch (j + j_offset)
-        done
-      done
-    | Vcat (t1, t2, _) ->
-      aux t1 j_offset;
-      aux t2 (j_offset + height t1)
-    | Hcat (t1, t2, _) ->
-      aux t1 j_offset;
-      aux t2 j_offset
-    | Ansi (prefix, t, suffix, _) ->
-      let vcopy s =
-        Option.iter s ~f:(fun s ->
-          for j = 0 to height t - 1 do
-            write_string s (j + j_offset)
-          done)
-      in
-      vcopy (Option.map ~f:Utf8_text.of_string prefix);
-      aux t j_offset;
-      vcopy (Option.map ~f:Utf8_text.of_string suffix)
+  let write_string txt j = String.Utf8.iter txt ~f:(fun uchar -> add_char uchar j) in
+  (* [aux] is written in continuation passing style to avoid stack overflows
+     on really big textblocks. https://en.wikipedia.org/wiki/Continuation-passing_style.
+     Typically continuation-passing style uses function closures as the continuation, but
+     here we defunctionalize the closure to get around js_of_ocaml's inability to do 
+     tail-call optimization in general. *)
+  let module K = struct
+    type k =
+      | Done
+      | Job of t * int * k
+      | Suffix of t * int * string * k
+  end
   in
-  aux t 0
+  let vcopy t j_offset s =
+    let s = String.Utf8.of_string s in
+    for j = 0 to height t - 1 do
+      write_string s (j + j_offset)
+    done
+  in
+  let rec aux = function
+    | K.Done -> ()
+    | Job (t, j_offset, k) ->
+      (match t with
+       | Text s ->
+         write_string s j_offset;
+         aux k
+       | Fill (ch, d) ->
+         for _i = 0 to d.width - 1 do
+           for j = 0 to d.height - 1 do
+             add_char ch (j + j_offset)
+           done
+         done;
+         aux k
+       | Vcat (t1, t2, _) -> aux (Job (t1, j_offset, Job (t2, j_offset + height t1, k)))
+       | Hcat (t1, t2, _) -> aux (Job (t1, j_offset, Job (t2, j_offset, k)))
+       | Ansi (prefix, t, suffix, _) ->
+         Option.iter ~f:(vcopy t j_offset) prefix;
+         let k =
+           match suffix with
+           | Some suffix -> K.Suffix (t, j_offset, suffix, k)
+           | None -> k
+         in
+         aux (Job (t, j_offset, k)))
+    | Suffix (t, j_offset, suffix, k) ->
+      vcopy t j_offset suffix;
+      aux k
+  in
+  aux (Job (t, 0, Done))
 ;;
 
 let line_lengths t =
@@ -415,12 +438,12 @@ module Up_or_down = struct
 end
 
 let span_banner
-      ~extend_left
-      ~extend_right
-      ~length
-      ~points:(up_or_down : Up_or_down.t)
-      ?label
-      ()
+  ~extend_left
+  ~extend_right
+  ~length
+  ~points:(up_or_down : Up_or_down.t)
+  ?label
+  ()
   =
   let up, down =
     match up_or_down with
@@ -444,8 +467,8 @@ let span_banner
   in
   vcat
     [ (match up_or_down with
-        | Down -> label_line
-        | Up -> nil)
+       | Down -> label_line
+       | Up -> nil)
     ; hcat
         (List.init length ~f:(fun i ->
            fill_uchar
@@ -549,17 +572,17 @@ module Boxed = struct
   let concat_frills project width_or_height ~ts ~n =
     List.init ((2 * n) - 1) ~f:Fn.id
     |> List.fold ~init:(0, []) ~f:(fun (sum, vals) i ->
-      let sum, new_vals =
-        if i % 2 = 0
-        then (
-          let t = ts.(i / 2) in
-          let vals = List.map (project t) ~f:(fun j -> j + sum) in
-          sum + width_or_height t.contents, vals)
-        else (
-          let vals = [ sum ] in
-          sum + 1, vals)
-      in
-      sum, List.rev_append new_vals vals)
+         let sum, new_vals =
+           if i % 2 = 0
+           then (
+             let t = ts.(i / 2) in
+             let vals = List.map (project t) ~f:(fun j -> j + sum) in
+             sum + width_or_height t.contents, vals)
+           else (
+             let vals = [ sum ] in
+             sum + 1, vals)
+         in
+         sum, List.rev_append new_vals vals)
     |> snd
     |> List.rev
   ;;
@@ -747,14 +770,14 @@ module List_with_static_lengths = struct
 
   let rec of_same_length_list_exn : type a shape. (a, shape) t -> a list -> (a, shape) t =
     fun t list ->
-      match t with
-      | [] ->
-        if not (List.is_empty list) then failwith "list is too long";
-        []
-      | _ :: t_tl ->
-        (match list with
-         | [] -> failwith "list is too short"
-         | list_hd :: list_tl -> list_hd :: of_same_length_list_exn t_tl list_tl)
+    match t with
+    | [] ->
+      if not (List.is_empty list) then failwith "list is too long";
+      []
+    | _ :: t_tl ->
+      (match list with
+       | [] -> failwith "list is too short"
+       | list_hd :: list_tl -> list_hd :: of_same_length_list_exn t_tl list_tl)
   ;;
 end
 
