@@ -94,28 +94,22 @@ let%expect_test "example" =
 
                                             Kel Varnsen
                                             Chief Procurement Officer
-  |}]
+    |}]
 ;;
 
 let%expect_test _ =
   test (hcat yoyoma);
-  [%expect {|
-    yoyoma
-  |}]
+  [%expect {| yoyoma |}]
 ;;
 
 let%expect_test _ =
   test (hcat ~sep:(hstrut 1) yoyoma);
-  [%expect {|
-    yo yo ma
-  |}]
+  [%expect {| yo yo ma |}]
 ;;
 
 let%expect_test _ =
   test (hcat ~sep:(hstrut 2) yoyoma);
-  [%expect {|
-    yo  yo  ma
-  |}]
+  [%expect {| yo  yo  ma |}]
 ;;
 
 let%expect_test _ =
@@ -124,7 +118,7 @@ let%expect_test _ =
     yo
     yo
     ma
-  |}]
+    |}]
 ;;
 
 let%expect_test _ =
@@ -135,7 +129,7 @@ let%expect_test _ =
     yo
 
     ma
-  |}]
+    |}]
 ;;
 
 let%expect_test _ =
@@ -148,7 +142,7 @@ let%expect_test _ =
 
 
     ma
-  |}]
+    |}]
 ;;
 
 let sep = text "."
@@ -159,7 +153,7 @@ let%expect_test _ =
     yo.yoyoma
     yo
     ma
-  |}]
+    |}]
 ;;
 
 let%expect_test _ =
@@ -168,7 +162,7 @@ let%expect_test _ =
     yoyoma.yo
            yo
            ma
-  |}]
+    |}]
 ;;
 
 let%expect_test _ =
@@ -179,7 +173,7 @@ let%expect_test _ =
     ma
     .
     yoyoma
-  |}]
+    |}]
 ;;
 
 let%expect_test _ =
@@ -190,7 +184,7 @@ let%expect_test _ =
     yo
     yo
     ma
-  |}]
+    |}]
 ;;
 
 let%expect_test "word wrap" =
@@ -229,7 +223,7 @@ let%expect_test "word wrap" =
     |officia deserunt mollit anim  |
     |id est laborum.               |
     +------------------------------+
-  |}];
+    |}];
   test
     ~width:33
     "(Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor \
@@ -257,7 +251,7 @@ let%expect_test "word wrap" =
     |officia deserunt mollit anim id  |
     |est laborum.)                    |
     +---------------------------------+
-  |}];
+    |}];
   test
     ~width:70
     "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor \
@@ -273,7 +267,7 @@ let%expect_test "word wrap" =
     |Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris    |
     |nisi ut aliquip ex ea commodo consequat.                              |
     +----------------------------------------------------------------------+
-  |}]
+    |}]
 ;;
 
 let%expect_test "compress_table_header" =
@@ -293,7 +287,8 @@ let%expect_test "compress_table_header" =
     |    |     |
       1  123   12345
      23  4567   678
-    456  89      9 |}]
+    456  89      9
+    |}]
 ;;
 
 (* Here's a trick for labeling an x-axis in an ascii graph by calling
@@ -331,7 +326,8 @@ let%expect_test "number markers" =
         | 398                 40      251
         | | 631               | 63    | 398
     100 | | | 1  2   5  10 20 | | 100 | |  794
-    |   | | | |  |   |  |  |  | | |   | |  | |}];
+    |   | | | |  |   |  |  |  | | |   | |  |
+    |}];
   ()
 ;;
 
@@ -342,7 +338,7 @@ let%expect_test _ =
   [%expect {|
     a
     b
-  |}]
+    |}]
 ;;
 
 let%expect_test _ =
@@ -350,7 +346,7 @@ let%expect_test _ =
   [%expect {|
     a
     b
-  |}]
+    |}]
 ;;
 
 let yellow = ansi_escape ~prefix:"[33m" ~suffix:"[39m"
@@ -361,7 +357,7 @@ let%expect_test _ =
     [33myo[39m
     [33myo[39m
     [33mma[39m
-  |}]
+    |}]
 ;;
 
 let%expect_test "unicode" =
@@ -377,7 +373,8 @@ let%expect_test "unicode" =
        ]);
   [%expect {|
     | ✓ yes |
-    | x no  | |}]
+    | x no  |
+    |}]
 ;;
 
 let%expect_test "fill_uchar" =
@@ -386,7 +383,8 @@ let%expect_test "fill_uchar" =
     😀😀
     😀😀
     😀😀
-    😀😀 |}]
+    😀😀
+    |}]
 ;;
 
 (* [Test_boxed] tests *)
@@ -401,10 +399,10 @@ module _ = struct
   let%expect_test "Basics" =
     dump Boxed.(hcat [ cell a; cell b ]);
     [%expect {|
-    ┌───┬───┐
-    │ A │ B │
-    └───┴───┘
-  |}];
+      ┌───┬───┐
+      │ A │ B │
+      └───┴───┘
+      |}];
     dump Boxed.(hcat ~align:`Center [ vcat [ cell a; cell b ]; cell c ]);
     dump
       Boxed.(
@@ -412,22 +410,22 @@ module _ = struct
     dump Boxed.(hcat ~align:`Center [ vcat [ cell a; cell b ]; vcat [ cell d; cell e ] ]);
     [%expect
       {|
-    ┌───┬───┐
-    │ A │   │
-    ├───┤ C │
-    │ B │   │
-    └───┴───┘
-    ┌───┬───┬───┐
-    │ A │   │ D │
-    ├───┤ C ├───┤
-    │ B │   │ E │
-    └───┴───┴───┘
-    ┌───┬───┐
-    │ A │ D │
-    ├───┼───┤
-    │ B │ E │
-    └───┴───┘
-  |}];
+      ┌───┬───┐
+      │ A │   │
+      ├───┤ C │
+      │ B │   │
+      └───┴───┘
+      ┌───┬───┬───┐
+      │ A │   │ D │
+      ├───┤ C ├───┤
+      │ B │   │ E │
+      └───┴───┴───┘
+      ┌───┬───┐
+      │ A │ D │
+      ├───┼───┤
+      │ B │ E │
+      └───┴───┘
+      |}];
     dump Boxed.(vcat ~align:`Center [ hcat [ cell a; cell b ]; cell c ]);
     dump
       Boxed.(
@@ -435,24 +433,24 @@ module _ = struct
     dump Boxed.(vcat ~align:`Center [ hcat [ cell a; cell b ]; hcat [ cell d; cell e ] ]);
     [%expect
       {|
-    ┌───┬───┐
-    │ A │ B │
-    ├───┴───┤
-    │   C   │
-    └───────┘
-    ┌───┬───┐
-    │ A │ B │
-    ├───┴───┤
-    │   C   │
-    ├───┬───┤
-    │ D │ E │
-    └───┴───┘
-    ┌───┬───┐
-    │ A │ B │
-    ├───┼───┤
-    │ D │ E │
-    └───┴───┘
-  |}]
+      ┌───┬───┐
+      │ A │ B │
+      ├───┴───┤
+      │   C   │
+      └───────┘
+      ┌───┬───┐
+      │ A │ B │
+      ├───┴───┤
+      │   C   │
+      ├───┬───┤
+      │ D │ E │
+      └───┴───┘
+      ┌───┬───┐
+      │ A │ B │
+      ├───┼───┤
+      │ D │ E │
+      └───┴───┘
+      |}]
   ;;
 
   let%expect_test "frills are correctly offset for padding" =
@@ -467,13 +465,14 @@ module _ = struct
           ]);
     [%expect
       {|
-    ┌───┬───┬──────────────┐
-    │ A │ B │              │
-    ├───┴───┤  Top right   │
-    │   C   ├──────────────┤
-    ├───┬───┤ Bottom right │
-    │ D │ E │              │
-    └───┴───┴──────────────┘ |}]
+      ┌───┬───┬──────────────┐
+      │ A │ B │              │
+      ├───┴───┤  Top right   │
+      │   C   ├──────────────┤
+      ├───┬───┤ Bottom right │
+      │ D │ E │              │
+      └───┴───┴──────────────┘
+      |}]
   ;;
 
   let%expect_test "align" =
@@ -505,22 +504,23 @@ module _ = struct
     test ~dir:`Horizontal;
     [%expect
       {|
-    ┌────────────┬──────────────────────┐
-    │ A Streeter │ 2½ Devonshire Square │
-    │            │ London               │
-    │            │ EC2M 4UJ             │
-    └────────────┴──────────────────────┘
-    ┌────────────┬────────────────┐
-    │            │ Windsor Castle │
-    │ Henry VIII │ Windsor        │
-    │            │ SL4 1NJ        │
-    └────────────┴────────────────┘
-    ┌───────────┬─────────────────────────┐
-    │           │ The White House         │
-    │           │ 1600 Pennsylvania Av NW │
-    │           │ Washington, DC          │
-    │ A Lincoln │ 20500                   │
-    └───────────┴─────────────────────────┘ |}];
+      ┌────────────┬──────────────────────┐
+      │ A Streeter │ 2½ Devonshire Square │
+      │            │ London               │
+      │            │ EC2M 4UJ             │
+      └────────────┴──────────────────────┘
+      ┌────────────┬────────────────┐
+      │            │ Windsor Castle │
+      │ Henry VIII │ Windsor        │
+      │            │ SL4 1NJ        │
+      └────────────┴────────────────┘
+      ┌───────────┬─────────────────────────┐
+      │           │ The White House         │
+      │           │ 1600 Pennsylvania Av NW │
+      │           │ Washington, DC          │
+      │ A Lincoln │ 20500                   │
+      └───────────┴─────────────────────────┘
+      |}];
     test ~dir:`Vertical;
     [%expect
       {|
@@ -545,7 +545,8 @@ module _ = struct
       │ 1600 Pennsylvania Av NW │
       │ Washington, DC          │
       │ 20500                   │
-      └─────────────────────────┘ |}]
+      └─────────────────────────┘
+      |}]
   ;;
 
   let%expect_test "fib" =
@@ -565,48 +566,48 @@ module _ = struct
     dump (nested_boxes [ 2; 1 ]);
     [%expect
       {|
-    ┌───────┬───┐
-    │       │   │
-    │       ├───┤
-    │       │   │
-    └───────┴───┘
-  |}];
+      ┌───────┬───┐
+      │       │   │
+      │       ├───┤
+      │       │   │
+      └───────┴───┘
+      |}];
     dump (nested_boxes [ 5; 3; 2; 1 ]);
     [%expect
       {|
-    ┌───────────────────┬───────────┐
-    │                   │           │
-    │                   │           │
-    │                   │           │
-    │                   │           │
-    │                   │           │
-    │                   ├───────┬───┤
-    │                   │       │   │
-    │                   │       ├───┤
-    │                   │       │   │
-    └───────────────────┴───────┴───┘
-  |}];
+      ┌───────────────────┬───────────┐
+      │                   │           │
+      │                   │           │
+      │                   │           │
+      │                   │           │
+      │                   │           │
+      │                   ├───────┬───┤
+      │                   │       │   │
+      │                   │       ├───┤
+      │                   │       │   │
+      └───────────────────┴───────┴───┘
+      |}];
     dump (nested_boxes [ 8; 5; 3; 2; 1 ]);
     [%expect
       {|
-    ┌───────────────────────────────┬───────────────────┐
-    │                               │                   │
-    │                               │                   │
-    │                               │                   │
-    │                               │                   │
-    │                               │                   │
-    │                               │                   │
-    │                               │                   │
-    │                               │                   │
-    │                               │                   │
-    │                               ├───────────┬───────┤
-    │                               │           │       │
-    │                               │           │       │
-    │                               │           │       │
-    │                               │           ├───┬───┤
-    │                               │           │   │   │
-    └───────────────────────────────┴───────────┴───┴───┘
-  |}]
+      ┌───────────────────────────────┬───────────────────┐
+      │                               │                   │
+      │                               │                   │
+      │                               │                   │
+      │                               │                   │
+      │                               │                   │
+      │                               │                   │
+      │                               │                   │
+      │                               │                   │
+      │                               │                   │
+      │                               ├───────────┬───────┤
+      │                               │           │       │
+      │                               │           │       │
+      │                               │           │       │
+      │                               │           ├───┬───┤
+      │                               │           │   │   │
+      └───────────────────────────────┴───────────┴───┴───┘
+      |}]
   ;;
 
   let%expect_test "padding" =
@@ -647,7 +648,8 @@ module _ = struct
       │     │ A │ B ││        │   │   ││        │   │   │
       │ Top │   │   ││ Center │ A │ B ││ Bottom │   │   │
       │     │   │   ││        │   │   ││        │ A │ B │
-      └─────┴───┴───┘└────────┴───┴───┘└────────┴───┴───┘ |}]
+      └─────┴───┴───┘└────────┴───┴───┘└────────┴───┴───┘
+      |}]
   ;;
 end
 
@@ -664,33 +666,33 @@ let%expect_test "span_banner" =
   let `Header header, `Rows rows =
     compress_table_header
       (`Cols
-         [ ( text "extend_left"
-           , List.map rows ~f:(fun (extend_left, _, _) ->
-               sexp [%sexp_of: bool] extend_left)
-           , `Left )
-         ; ( text "extend_right"
-           , List.map rows ~f:(fun (_, extend_right, _) ->
-               sexp [%sexp_of: bool] extend_right)
-           , `Left )
-         ; ( text "points"
-           , List.map rows ~f:(fun (_, _, points) -> sexp [%sexp_of: Up_or_down.t] points)
-           , `Left )
-         ; ( text "span_banner"
-           , List.map rows ~f:(fun (extend_left, extend_right, points) ->
-               let align =
-                 match points with
-                 | Up -> `Top
-                 | Down -> `Bottom
-               in
-               hcat
-                 ~align
-                 [ vstrut 2; span_banner ~extend_left ~extend_right ~points ~length () ])
-           , `Left )
-         ; ( text "span_banner ~label"
-           , List.map rows ~f:(fun (extend_left, extend_right, points) ->
-               span_banner ~extend_left ~extend_right ~points ~length ~label ())
-           , `Left )
-         ])
+        [ ( text "extend_left"
+          , List.map rows ~f:(fun (extend_left, _, _) ->
+              sexp [%sexp_of: bool] extend_left)
+          , `Left )
+        ; ( text "extend_right"
+          , List.map rows ~f:(fun (_, extend_right, _) ->
+              sexp [%sexp_of: bool] extend_right)
+          , `Left )
+        ; ( text "points"
+          , List.map rows ~f:(fun (_, _, points) -> sexp [%sexp_of: Up_or_down.t] points)
+          , `Left )
+        ; ( text "span_banner"
+          , List.map rows ~f:(fun (extend_left, extend_right, points) ->
+              let align =
+                match points with
+                | Up -> `Top
+                | Down -> `Bottom
+              in
+              hcat
+                ~align
+                [ vstrut 2; span_banner ~extend_left ~extend_right ~points ~length () ])
+          , `Left )
+        ; ( text "span_banner ~label"
+          , List.map rows ~f:(fun (extend_left, extend_right, points) ->
+              span_banner ~extend_left ~extend_right ~points ~length ~label ())
+          , `Left )
+        ])
   in
   test (vcat ~sep:vsep (header :: rows));
   [%expect
@@ -724,6 +726,13 @@ let%expect_test "span_banner" =
                         ─────────┐  ─┴───────┐
 
     true   true   Down               label
-                        ──────────  ─┴──────── |}];
+                        ──────────  ─┴────────
+    |}];
   ()
 ;;
+
+let%expect_test "Regression test against stack overflows" =
+  let textblock = Text_block.vcat (List.init 1_000_000 ~f:(Fn.const (text ""))) in
+  Expect_test_helpers_core.require_does_not_raise [%here] (fun () ->
+    Text_block.render textblock |> (ignore : string -> unit))
+;;
